@@ -7,11 +7,9 @@ Main function.
 from __future__ import annotations
 
 import json
-import locale
 import signal
 import sys
 import warnings
-from argparse import ArgumentParser, FileType, Namespace
 from collections import defaultdict
 from enum import IntEnum
 from io import TextIOBase
@@ -33,11 +31,14 @@ from urllib.parse import ParseResult, urlparse
 
 import argcomplete
 import black
-import toml
 from pydantic import BaseModel
 
+if TYPE_CHECKING:
+    from argparse import Namespace
+
+    from typing_extensions import Self
+
 from datamodel_code_generator import (
-    DEFAULT_BASE_CLASS,
     DataModelType,
     Error,
     InputFileType,
@@ -46,6 +47,7 @@ from datamodel_code_generator import (
     enable_debug_message,
     generate,
 )
+from datamodel_code_generator.arguments import DEFAULT_ENCODING, arg_parser, namespace
 from datamodel_code_generator.format import (
     PythonVersion,
     black_find_project_root,
@@ -59,6 +61,7 @@ from datamodel_code_generator.util import (
     ConfigDict,
     Model,
     field_validator,
+    load_toml,
     model_validator,
 )
 
@@ -77,388 +80,6 @@ def sig_int_handler(_: int, __: Any) -> None:  # pragma: no cover
 
 signal.signal(signal.SIGINT, sig_int_handler)
 
-DEFAULT_ENCODING = locale.getpreferredencoding()
-
-arg_parser = ArgumentParser()
-arg_parser.add_argument(
-    '--input',
-    help='Input file/directory (default: stdin)',
-)
-arg_parser.add_argument(
-    '--url',
-    help='Input file URL. `--input` is ignored when `--url` is used',
-)
-
-arg_parser.add_argument(
-    '--http-headers',
-    nargs='+',
-    metavar='HTTP_HEADER',
-    help='Set headers in HTTP requests to the remote host. (example: "Authorization: Basic dXNlcjpwYXNz")',
-)
-
-arg_parser.add_argument(
-    '--http-ignore-tls',
-    help="Disable verification of the remote host's TLS certificate",
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--input-file-type',
-    help='Input file type (default: auto)',
-    choices=[i.value for i in InputFileType],
-)
-arg_parser.add_argument(
-    '--output-model-type',
-    help='Output model type (default: pydantic.BaseModel)',
-    choices=[i.value for i in DataModelType],
-)
-arg_parser.add_argument(
-    '--openapi-scopes',
-    help='Scopes of OpenAPI model generation (default: schemas)',
-    choices=[o.value for o in OpenAPIScope],
-    nargs='+',
-    default=None,
-)
-arg_parser.add_argument('--output', help='Output file (default: stdout)')
-
-arg_parser.add_argument(
-    '--base-class',
-    help='Base Class (default: pydantic.BaseModel)',
-    type=str,
-)
-arg_parser.add_argument(
-    '--field-constraints',
-    help='Use field constraints and not con* annotations',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--use-annotated',
-    help='Use typing.Annotated for Field(). Also, `--field-constraints` option will be enabled.',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--use-non-positive-negative-number-constrained-types',
-    help='Use the Non{Positive,Negative}{FloatInt} types instead of the corresponding con* constrained types.',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--field-extra-keys',
-    help='Add extra keys to field parameters',
-    type=str,
-    nargs='+',
-)
-arg_parser.add_argument(
-    '--field-include-all-keys',
-    help='Add all keys to field parameters',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--field-extra-keys-without-x-prefix',
-    help='Add extra keys with `x-` prefix to field parameters. The extra keys are stripped of the `x-` prefix.',
-    type=str,
-    nargs='+',
-)
-arg_parser.add_argument(
-    '--snake-case-field',
-    help='Change camel-case field name to snake-case',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--original-field-name-delimiter',
-    help='Set delimiter to convert to snake case. This option only can be used with --snake-case-field (default: `_` )',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--strip-default-none',
-    help='Strip default None on fields',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--disable-appending-item-suffix',
-    help='Disable appending `Item` suffix to model name in an array',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--allow-population-by-field-name',
-    help='Allow population by field name',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--allow-extra-fields',
-    help='Allow to pass extra fields, if this flag is not passed, extra fields are forbidden.',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--enable-faux-immutability',
-    help='Enable faux immutability',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-default',
-    help='Use default value even if a field is required',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--force-optional',
-    help='Force optional for required fields',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--strict-nullable',
-    help='Treat default field as a non-nullable field (Only OpenAPI)',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--strict-types',
-    help='Use strict types',
-    choices=[t.value for t in StrictTypes],
-    nargs='+',
-)
-
-arg_parser.add_argument(
-    '--disable-timestamp',
-    help='Disable timestamp on file headers',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--enable-version-header',
-    help='Enable package version on file headers',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-standard-collections',
-    help='Use standard collections for type hinting (list, dict)',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-generic-container-types',
-    help='Use generic container types for type hinting (typing.Sequence, typing.Mapping). '
-    'If `--use-standard-collections` option is set, then import from collections.abc instead of typing',
-    action='store_true',
-    default=None,
-)
-arg_parser.add_argument(
-    '--use-union-operator',
-    help='Use | operator for Union type (PEP 604).',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-schema-description',
-    help='Use schema description to populate class docstring',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-field-description',
-    help='Use schema description to populate field docstring',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-default-kwarg',
-    action='store_true',
-    help='Use `default=` instead of a positional argument for Fields that have default values.',
-)
-
-arg_parser.add_argument(
-    '--reuse-model',
-    help='Re-use models on the field when a module has the model with the same content',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--keep-model-order',
-    help="Keep generated models' order",
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--collapse-root-models',
-    action='store_true',
-    default=None,
-    help='Models generated with a root-type field will be merged'
-    'into the models using that root-type model',
-)
-
-
-arg_parser.add_argument(
-    '--enum-field-as-literal',
-    help='Parse enum field as literal. '
-    'all: all enum field type are Literal. '
-    'one: field type is Literal when an enum has only one possible value',
-    choices=[lt.value for lt in LiteralType],
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-one-literal-as-default',
-    help='Use one literal as default value for one literal field',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--set-default-enum-member',
-    help='Set enum members as default values for enum field',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--empty-enum-field-name',
-    help='Set field name when enum value is empty (default:  `_`)',
-    default=None,
-)
-
-
-arg_parser.add_argument(
-    '--capitalise-enum-members',
-    help='Capitalize field names on enum',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--special-field-name-prefix',
-    help="Set field name prefix when first character can't be used as Python field name (default:  `field`)",
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--remove-special-field-name-prefix',
-    help="Remove field name prefix when first character can't be used as Python field name",
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-subclass-enum',
-    help='Define Enum class as subclass with field type when enum has type (int, float, bytes, str)',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--class-name',
-    help='Set class name of root model',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-title-as-name',
-    help='use titles as class names of models',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-operation-id-as-name',
-    help='use operation id of OpenAPI as class names of models',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-unique-items-as-set',
-    help='define field type as `set` when the field attribute has `uniqueItems`',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--custom-template-dir', help='Custom template directory', type=str
-)
-arg_parser.add_argument(
-    '--extra-template-data', help='Extra template data', type=FileType('rt')
-)
-arg_parser.add_argument('--aliases', help='Alias mapping file', type=FileType('rt'))
-arg_parser.add_argument(
-    '--target-python-version',
-    help='target python version (default: 3.7)',
-    choices=[v.value for v in PythonVersion],
-)
-
-arg_parser.add_argument(
-    '--wrap-string-literal',
-    help='Wrap string literal by using black `experimental-string-processing` option (require black 20.8b0 or later)',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--validation',
-    help='Enable validation (Only OpenAPI)',
-    action='store_true',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--use-double-quotes',
-    action='store_true',
-    default=None,
-    help='Model generated with double quotes. Single quotes or '
-    'your black config skip_string_normalization value will be used without this option.',
-)
-
-arg_parser.add_argument(
-    '--encoding',
-    help=f'The encoding of input and output (default: {DEFAULT_ENCODING})',
-    default=None,
-)
-
-arg_parser.add_argument(
-    '--debug', help='show debug message', action='store_true', default=None
-)
-arg_parser.add_argument(
-    '--disable-warnings', help='disable warnings', action='store_true', default=None
-)
-arg_parser.add_argument(
-    '--custom-file-header', help='Custom file header', type=str, default=None
-)
-
-arg_parser.add_argument(
-    '--custom-file-header-path',
-    help='Custom file header file path',
-    default=None,
-    type=str,
-)
-
-
-arg_parser.add_argument('--version', help='show version', action='store_true')
-
 
 class Config(BaseModel):
     if PYDANTIC_V2:
@@ -470,9 +91,6 @@ class Config(BaseModel):
         def __getitem__(self, item: str) -> Any:
             return self.get(item)
 
-        def __setitem__(self, key: str, value: Any) -> None:
-            setattr(self, key, value)
-
         if TYPE_CHECKING:
 
             @classmethod
@@ -502,7 +120,9 @@ class Config(BaseModel):
             def get_fields(cls) -> Dict[str, Any]:
                 return cls.__fields__
 
-    @field_validator('aliases', 'extra_template_data', mode='before')
+    @field_validator(
+        'aliases', 'extra_template_data', 'custom_formatters_kwargs', mode='before'
+    )
     def validate_file(cls, value: Any) -> Optional[TextIOBase]:
         if value is None or isinstance(value, TextIOBase):
             return value
@@ -568,9 +188,7 @@ class Config(BaseModel):
         def validate_each_item(each_item: Any) -> Tuple[str, str]:
             if isinstance(each_item, str):  # pragma: no cover
                 try:
-                    field_name, field_value = each_item.split(
-                        ':', maxsplit=1
-                    )  # type: str, str
+                    field_name, field_value = each_item.split(':', maxsplit=1)  # type: str, str
                     return field_name, field_value.lstrip()
                 except ValueError:
                     raise Error(f'Invalid http header: {each_item!r}')
@@ -580,24 +198,38 @@ class Config(BaseModel):
             return [validate_each_item(each_item) for each_item in value]
         return value  # pragma: no cover
 
-    @model_validator(mode='after')
-    def validate_root(cls, values: Dict[str, Any]) -> Dict[str, Any]:
-        values = cls._validate_use_annotated(values)
-        return cls._validate_base_class(values)
-
-    @classmethod
-    def _validate_use_annotated(cls, values: Dict[str, Any]) -> Dict[str, Any]:
-        if values.get('use_annotated'):
-            values['field_constraints'] = True
+    @model_validator(mode='before')
+    def validate_additional_imports(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        if values.get('additional_imports') is not None:
+            values['additional_imports'] = values.get('additional_imports').split(',')
+        else:
+            values['additional_imports'] = []
         return values
 
-    @classmethod
-    def _validate_base_class(cls, values: Dict[str, Any]) -> Dict[str, Any]:
-        if 'base_class' not in values and 'output_model_type' in values:
-            if values['output_model_type'] != DataModelType.PydanticBaseModel.value:
-                values['base_class'] = ''
+    @model_validator(mode='before')
+    def validate_custom_formatters(cls, values: Dict[str, Any]) -> Dict[str, Any]:
+        if values.get('custom_formatters') is not None:
+            values['custom_formatters'] = values.get('custom_formatters').split(',')
+        else:
+            values['custom_formatters'] = []
         return values
 
+    if PYDANTIC_V2:
+
+        @model_validator(mode='after')  # type: ignore
+        def validate_root(self: Self) -> Self:
+            if self.use_annotated:
+                self.field_constraints = True
+            return self
+
+    else:
+
+        @model_validator(mode='after')
+        def validate_root(cls, values: Any) -> Any:
+            if values.get('use_annotated'):
+                values['field_constraints'] = True
+            return values
+
     input: Optional[Union[Path, str]] = None
     input_file_type: InputFileType = InputFileType.Auto
     output_model_type: DataModelType = DataModelType.PydanticBaseModel
@@ -605,7 +237,8 @@ class Config(BaseModel):
     debug: bool = False
     disable_warnings: bool = False
     target_python_version: PythonVersion = PythonVersion.PY_37
-    base_class: str = DEFAULT_BASE_CLASS
+    base_class: str = ''
+    additional_imports: Optional[List[str]] = (None,)
     custom_template_dir: Optional[Path] = None
     extra_template_data: Optional[TextIOBase] = None
     validation: bool = False
@@ -623,7 +256,7 @@ class Config(BaseModel):
     use_standard_collections: bool = False
     use_schema_description: bool = False
     use_field_description: bool = False
-    use_default_kwarg: bool = True
+    use_default_kwarg: bool = False
     reuse_model: bool = False
     encoding: str = DEFAULT_ENCODING
     enum_field_as_literal: Optional[LiteralType] = None
@@ -659,6 +292,8 @@ class Config(BaseModel):
     keep_model_order: bool = False
     custom_file_header: Optional[str] = None
     custom_file_header_path: Optional[Path] = None
+    custom_formatters: Optional[List[str]] = None
+    custom_formatters_kwargs: Optional[TextIOBase] = None
 
     def merge_args(self, args: Namespace) -> None:
         set_args = {
@@ -666,9 +301,14 @@ class Config(BaseModel):
             for f in self.get_fields()
             if getattr(args, f) is not None
         }
-        set_args = self._validate_use_annotated(set_args)
-        set_args = self._validate_base_class(set_args)
-        parsed_args = self.parse_obj(set_args)
+
+        if set_args.get('output_model_type') == DataModelType.MsgspecStruct.value:
+            set_args['use_annotated'] = True
+
+        if set_args.get('use_annotated'):
+            set_args['field_constraints'] = True
+
+        parsed_args = Config.parse_obj(set_args)
         for field_name in set_args:
             setattr(self, field_name, getattr(parsed_args, field_name))
 
@@ -679,10 +319,10 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
     # add cli completion support
     argcomplete.autocomplete(arg_parser)
 
-    if args is None:
+    if args is None:  # pragma: no cover
         args = sys.argv[1:]
 
-    namespace: Namespace = arg_parser.parse_args(args)
+    arg_parser.parse_args(args, namespace=namespace)
 
     if namespace.version:
         from datamodel_code_generator.version import version
@@ -695,7 +335,7 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
     if pyproject_toml_path.is_file():
         pyproject_toml: Dict[str, Any] = {
             k.replace('-', '_'): v
-            for k, v in toml.load(str(pyproject_toml_path))
+            for k, v in load_toml(pyproject_toml_path)
             .get('tool', {})
             .get('datamodel-codegen', {})
             .items()
@@ -721,8 +361,8 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
     if not is_supported_in_black(config.target_python_version):  # pragma: no cover
         print(
             f"Installed black doesn't support Python version {config.target_python_version.value}.\n"  # type: ignore
-            f"You have to install a newer black.\n"
-            f"Installed black version: {black.__version__}",
+            f'You have to install a newer black.\n'
+            f'Installed black version: {black.__version__}',
             file=sys.stderr,
         )
         return Exit.ERROR
@@ -763,6 +403,28 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
             )
             return Exit.ERROR
 
+    if config.custom_formatters_kwargs is None:
+        custom_formatters_kwargs = None
+    else:
+        with config.custom_formatters_kwargs as data:
+            try:
+                custom_formatters_kwargs = json.load(data)
+            except json.JSONDecodeError as e:
+                print(
+                    f'Unable to load custom_formatters_kwargs mapping: {e}',
+                    file=sys.stderr,
+                )
+                return Exit.ERROR
+        if not isinstance(custom_formatters_kwargs, dict) or not all(
+            isinstance(k, str) and isinstance(v, str)
+            for k, v in custom_formatters_kwargs.items()
+        ):
+            print(
+                'Custom formatters kwargs mapping must be a JSON string mapping (e.g. {"from": "to", ...})',
+                file=sys.stderr,
+            )
+            return Exit.ERROR
+
     try:
         generate(
             input_=config.url or config.input or sys.stdin.read(),
@@ -771,6 +433,7 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
             output_model_type=config.output_model_type,
             target_python_version=config.target_python_version,
             base_class=config.base_class,
+            additional_imports=config.additional_imports,
             custom_template_dir=config.custom_template_dir,
             validation=config.validation,
             field_constraints=config.field_constraints,
@@ -823,6 +486,8 @@ def main(args: Optional[Sequence[str]] = None) -> Exit:
             keep_model_order=config.keep_model_order,
             custom_file_header=config.custom_file_header,
             custom_file_header_path=config.custom_file_header_path,
+            custom_formatters=config.custom_formatters,
+            custom_formatters_kwargs=custom_formatters_kwargs,
         )
         return Exit.OK
     except InvalidClassNameError as e:
