@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 from abc import ABC, abstractmethod
 from collections import defaultdict
 from functools import lru_cache
@@ -16,6 +14,7 @@ from typing import (
     Optional,
     Set,
     Tuple,
+    TypeVar,
     Union,
 )
 from warnings import warn
@@ -46,6 +45,8 @@ TEMPLATE_DIR: Path = Path(__file__).parents[0] / 'template'
 
 ALL_MODEL: str = '#all#'
 
+ConstraintsBaseT = TypeVar('ConstraintsBaseT', bound='ConstraintsBase')
+
 
 class ConstraintsBase(_BaseModel):
     unique_items: Optional[bool] = Field(None, alias='uniqueItems')
@@ -64,6 +65,37 @@ class ConstraintsBase(_BaseModel):
     def has_constraints(self) -> bool:
         return any(v for v in self.dict().values() if v is not None)
 
+    @staticmethod
+    def merge_constraints(
+        a: ConstraintsBaseT, b: ConstraintsBaseT
+    ) -> Optional[ConstraintsBaseT]:
+        constraints_class = None
+        if isinstance(a, ConstraintsBase):  # pragma: no cover
+            root_type_field_constraints = {
+                k: v for k, v in a.dict(by_alias=True).items() if v is not None
+            }
+            constraints_class = a.__class__
+        else:
+            root_type_field_constraints = {}
+
+        if isinstance(b, ConstraintsBase):  # pragma: no cover
+            model_field_constraints = {
+                k: v for k, v in b.dict(by_alias=True).items() if v is not None
+            }
+            constraints_class = constraints_class or b.__class__
+        else:
+            model_field_constraints = {}
+
+        if not issubclass(constraints_class, ConstraintsBase):
+            return None
+
+        return constraints_class.parse_obj(
+            {
+                **root_type_field_constraints,
+                **model_field_constraints,
+            }
+        )
+
 
 class DataModelFieldBase(_BaseModel):
     name: Optional[str] = None
@@ -85,6 +117,7 @@ class DataModelFieldBase(_BaseModel):
     use_one_literal_as_default: bool = False
     _exclude_fields: ClassVar[Set[str]] = {'parent'}
     _pass_fields: ClassVar[Set[str]] = {'parent', 'data_type'}
+    can_have_extra_keys: ClassVar[bool] = True
 
     if not TYPE_CHECKING:
 
@@ -92,11 +125,15 @@ class DataModelFieldBase(_BaseModel):
             super().__init__(**data)
             if self.data_type.reference or self.data_type.data_types:
                 self.data_type.parent = self
-            if 'const' in self.extras:
-                self.default = self.extras['const']
-                self.const = True
-                self.required = False
-                self.nullable = False
+            self.process_const()
+
+    def process_const(self) -> None:
+        if 'const' not in self.extras:
+            return None
+        self.default = self.extras['const']
+        self.const = True
+        self.required = False
+        self.nullable = False
 
     @property
     def type_hint(self) -> str:
@@ -114,7 +151,10 @@ class DataModelFieldBase(_BaseModel):
             return type_hint
         elif self.required:
             return type_hint
-        return get_optional_type(type_hint, self.data_type.use_union_operator)
+        elif self.fall_back_to_nullable:
+            return get_optional_type(type_hint, self.data_type.use_union_operator)
+        else:
+            return type_hint
 
     @property
     def imports(self) -> Tuple[Import, ...]:
@@ -128,11 +168,17 @@ class DataModelFieldBase(_BaseModel):
             )
         ]
 
-        if (
-            self.nullable or (self.nullable is None and not self.required)
-        ) and not self.data_type.use_union_operator:
-            imports.append((IMPORT_OPTIONAL,))
-        if self.use_annotated:
+        if self.fall_back_to_nullable:
+            if (
+                self.nullable or (self.nullable is None and not self.required)
+            ) and not self.data_type.use_union_operator:
+                imports.append((IMPORT_OPTIONAL,))
+        else:
+            if (
+                self.nullable and not self.data_type.use_union_operator
+            ):  # pragma: no cover
+                imports.append((IMPORT_OPTIONAL,))
+        if self.use_annotated and self.annotated:
             import_annotated = (
                 IMPORT_ANNOTATED
                 if self.data_type.python_version.has_annotated_type
@@ -174,6 +220,10 @@ class DataModelFieldBase(_BaseModel):
     def has_default_factory(self) -> bool:
         return 'default_factory' in self.extras
 
+    @property
+    def fall_back_to_nullable(self) -> bool:
+        return True
+
 
 @lru_cache()
 def get_template(template_file_path: Path) -> Template:
