@@ -19,6 +19,7 @@ from typing import (
     Set,
     Tuple,
     Type,
+    TypeVar,
     Union,
 )
 from urllib.parse import ParseResult
@@ -26,9 +27,15 @@ from urllib.parse import ParseResult
 from pydantic import BaseModel
 
 from datamodel_code_generator.format import CodeFormatter, PythonVersion
-from datamodel_code_generator.imports import IMPORT_ANNOTATIONS, Import, Imports
+from datamodel_code_generator.imports import (
+    IMPORT_ANNOTATIONS,
+    IMPORT_LITERAL,
+    IMPORT_LITERAL_BACKPORT,
+    Import,
+    Imports,
+)
 from datamodel_code_generator.model import pydantic as pydantic_model
-from datamodel_code_generator.model import pydantic_v2 as pydantic_v2_model
+from datamodel_code_generator.model import pydantic_v2 as pydantic_model_v2
 from datamodel_code_generator.model.base import (
     ALL_MODEL,
     UNDEFINED,
@@ -43,10 +50,17 @@ from datamodel_code_generator.reference import ModelResolver, Reference
 from datamodel_code_generator.types import DataType, DataTypeManager, StrictTypes
 from datamodel_code_generator.util import Protocol, runtime_checkable
 
+SPECIAL_PATH_FORMAT: str = '#-datamodel-code-generator-#-{}-#-special-#'
+
+
+def get_special_path(keyword: str, path: List[str]) -> List[str]:
+    return [*path, SPECIAL_PATH_FORMAT.format(keyword)]
+
+
 escape_characters = str.maketrans(
     {
         '\\': r'\\',
-        "'": r"\'",
+        "'": r'\'',
         '\b': r'\b',
         '\f': r'\f',
         '\n': r'\n',
@@ -231,9 +245,12 @@ class Child(Protocol):
         raise NotImplementedError
 
 
-def get_most_of_parent(value: Any) -> Optional[Any]:
-    if isinstance(value, Child):
-        return get_most_of_parent(value.parent)
+T = TypeVar('T')
+
+
+def get_most_of_parent(value: Any, type_: Optional[Type[T]] = None) -> Optional[T]:
+    if isinstance(value, Child) and (type_ is None or not isinstance(value, type_)):
+        return get_most_of_parent(value.parent, type_)
     return value
 
 
@@ -314,6 +331,7 @@ class Parser(ABC):
         data_type_manager_type: Type[DataTypeManager] = pydantic_model.DataTypeManager,
         data_model_field_type: Type[DataModelFieldBase] = pydantic_model.DataModelField,
         base_class: Optional[str] = None,
+        additional_imports: Optional[List[str]] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
@@ -369,6 +387,9 @@ class Parser(ABC):
         capitalise_enum_members: bool = False,
         keep_model_order: bool = False,
         use_one_literal_as_default: bool = False,
+        known_third_party: Optional[List[str]] = None,
+        custom_formatters: Optional[List[str]] = None,
+        custom_formatters_kwargs: Optional[Dict[str, Any]] = None,
     ) -> None:
         self.data_type_manager: DataTypeManager = data_type_manager_type(
             python_version=target_python_version,
@@ -380,7 +401,10 @@ class Parser(ABC):
         self.data_model_type: Type[DataModel] = data_model_type
         self.data_model_root_type: Type[DataModel] = data_model_root_type
         self.data_model_field_type: Type[DataModelFieldBase] = data_model_field_type
+
         self.imports: Imports = Imports()
+        self._append_additional_imports(additional_imports=additional_imports)
+
         self.base_class: Optional[str] = base_class
         self.target_python_version: PythonVersion = target_python_version
         self.results: List[DataModel] = []
@@ -391,12 +415,12 @@ class Parser(ABC):
         self.field_constraints: bool = field_constraints
         self.snake_case_field: bool = snake_case_field
         self.strip_default_none: bool = strip_default_none
-        self.apply_default_values_for_required_fields: bool = (
-            apply_default_values_for_required_fields
-        )
-        self.force_optional_for_required_fields: bool = (
-            force_optional_for_required_fields
-        )
+        self.apply_default_values_for_required_fields: (
+            bool
+        ) = apply_default_values_for_required_fields
+        self.force_optional_for_required_fields: (
+            bool
+        ) = force_optional_for_required_fields
         self.use_schema_description: bool = use_schema_description
         self.use_field_description: bool = use_field_description
         self.use_default_kwarg: bool = use_default_kwarg
@@ -437,9 +461,9 @@ class Parser(ABC):
 
         self.source: Union[str, Path, List[Path], ParseResult] = source
         self.custom_template_dir = custom_template_dir
-        self.extra_template_data: DefaultDict[
-            str, Any
-        ] = extra_template_data or defaultdict(dict)
+        self.extra_template_data: DefaultDict[str, Any] = (
+            extra_template_data or defaultdict(dict)
+        )
 
         if allow_population_by_field_name:
             self.extra_template_data[ALL_MODEL]['allow_population_by_field_name'] = True
@@ -481,6 +505,9 @@ class Parser(ABC):
         self.capitalise_enum_members = capitalise_enum_members
         self.keep_model_order = keep_model_order
         self.use_one_literal_as_default = use_one_literal_as_default
+        self.known_third_party = known_third_party
+        self.custom_formatter = custom_formatters
+        self.custom_formatters_kwargs = custom_formatters_kwargs
 
     @property
     def iter_source(self) -> Iterator[Source]:
@@ -504,6 +531,16 @@ class Parser(ABC):
                 ),
             )
 
+    def _append_additional_imports(
+        self, additional_imports: Optional[List[str]]
+    ) -> None:
+        if additional_imports is None:
+            additional_imports = []
+
+        for additional_import_string in additional_imports:
+            new_import = Import.from_full_path(additional_import_string)
+            self.imports.append(new_import)
+
     def _get_text_from_url(self, url: str) -> str:
         from datamodel_code_generator.http import get_body
 
@@ -664,7 +701,14 @@ class Parser(ABC):
 
                 if init:
                     from_ = '.' + from_
-                imports.append(Import(from_=from_, import_=import_, alias=alias))
+                imports.append(
+                    Import(
+                        from_=from_,
+                        import_=import_,
+                        alias=alias,
+                        reference_path=data_type.reference.path,
+                    ),
+                )
 
     @classmethod
     def __extract_inherited_enum(cls, models: List[DataModel]) -> None:
@@ -689,6 +733,82 @@ class Parser(ABC):
                 )
                 models.remove(model)
 
+    def __apply_discriminator_type(
+        self,
+        models: List[DataModel],
+        imports: Imports,
+    ) -> None:
+        for model in models:
+            for field in model.fields:
+                discriminator = field.extras.get('discriminator')
+                if not discriminator or not isinstance(discriminator, dict):
+                    continue
+                property_name = discriminator.get('propertyName')
+                if not property_name:  # pragma: no cover
+                    continue
+                mapping = discriminator.get('mapping', {})
+                for data_type in field.data_type.data_types:
+                    if not data_type.reference:  # pragma: no cover
+                        continue
+                    discriminator_model = data_type.reference.source
+                    if not isinstance(  # pragma: no cover
+                        discriminator_model,
+                        (pydantic_model.BaseModel, pydantic_model_v2.BaseModel),
+                    ):
+                        continue  # pragma: no cover
+                    type_name = None
+                    if mapping:
+                        for name, path in mapping.items():
+                            if (
+                                discriminator_model.path.split('#/')[-1]
+                                != path.split('#/')[-1]
+                            ):
+                                # TODO: support external reference
+                                continue
+                            type_name = name
+                    else:
+                        type_name = discriminator_model.path.split('/')[-1]
+                    if not type_name:  # pragma: no cover
+                        raise RuntimeError(
+                            f'Discriminator type is not found. {data_type.reference.path}'
+                        )
+                    has_one_literal = False
+                    for discriminator_field in discriminator_model.fields:
+                        if (
+                            discriminator_field.original_name
+                            or discriminator_field.name
+                        ) != property_name:
+                            continue
+                        literals = discriminator_field.data_type.literals
+                        if len(literals) == 1 and literals[0] == type_name:
+                            has_one_literal = True
+                            continue
+                        for (
+                            field_data_type
+                        ) in discriminator_field.data_type.all_data_types:
+                            if field_data_type.reference:  # pragma: no cover
+                                field_data_type.remove_reference()
+                        discriminator_field.data_type = self.data_type(
+                            literals=[type_name]
+                        )
+                        discriminator_field.data_type.parent = discriminator_field
+                        discriminator_field.required = True
+                        imports.append(discriminator_field.imports)
+                        has_one_literal = True
+                    if not has_one_literal:
+                        discriminator_model.fields.append(
+                            self.data_model_field_type(
+                                name=property_name,
+                                data_type=self.data_type(literals=[type_name]),
+                                required=True,
+                            )
+                        )
+                    imports.append(
+                        IMPORT_LITERAL
+                        if self.target_python_version.has_literal_type
+                        else IMPORT_LITERAL_BACKPORT
+                    )
+
     @classmethod
     def _create_set_from_list(cls, data_type: DataType) -> Optional[DataType]:
         if data_type.is_list:
@@ -709,10 +829,7 @@ class Parser(ABC):
     def __replace_unique_list_to_set(self, models: List[DataModel]) -> None:
         for model in models:
             for model_field in model.fields:
-                if not (
-                    isinstance(model_field, pydantic_v2_model.DataModelField)
-                    or self.use_unique_items_as_set
-                ):
+                if not self.use_unique_items_as_set:
                     continue
 
                 if not (
@@ -783,10 +900,11 @@ class Parser(ABC):
             models.remove(duplicate)
 
     def __collapse_root_models(
-        self, models: List[DataModel], unused_models: List[DataModel]
+        self, models: List[DataModel], unused_models: List[DataModel], imports: Imports
     ) -> None:
         if not self.collapse_root_models:
             return None
+
         for model in models:
             for model_field in model.fields:
                 for data_type in model_field.data_type.all_data_types:
@@ -807,7 +925,7 @@ class Parser(ABC):
                         and any(
                             d
                             for d in model_field.data_type.all_data_types
-                            if d.is_dict or d.is_list or d.is_union
+                            if d.is_dict or d.is_union
                         )
                     ):
                         continue
@@ -817,29 +935,33 @@ class Parser(ABC):
                     if isinstance(data_type.parent, self.data_model_field_type):
                         # for field
                         # override empty field by root-type field
-                        model_field.extras = dict(
-                            root_type_field.extras, **model_field.extras
-                        )
+                        model_field.extras = {
+                            **root_type_field.extras,
+                            **model_field.extras,
+                        }
+                        model_field.process_const()
+
                         if self.field_constraints:
-                            if isinstance(
-                                root_type_field.constraints, ConstraintsBase
-                            ):  # pragma: no cover
-                                model_field.constraints = root_type_field.constraints.copy(
-                                    update={
-                                        k: v
-                                        for k, v in model_field.constraints.dict().items()
-                                        if v is not None
-                                    }
-                                    if isinstance(
-                                        model_field.constraints, ConstraintsBase
-                                    )
-                                    else {}
-                                )
-                        else:
-                            pass
-                            # skip function type-hint kwargs overriding
+                            model_field.constraints = ConstraintsBase.merge_constraints(
+                                root_type_field.constraints, model_field.constraints
+                            )
 
                         data_type.parent.data_type = copied_data_type
+
+                    elif data_type.parent.is_list:
+                        if self.field_constraints:
+                            model_field.constraints = ConstraintsBase.merge_constraints(
+                                root_type_field.constraints, model_field.constraints
+                            )
+                        if isinstance(
+                            root_type_field, pydantic_model.DataModelField
+                        ) and not model_field.extras.get('discriminator'):  # no: pragma
+                            discriminator = root_type_field.extras.get('discriminator')
+                            if discriminator:  # no: pragma
+                                model_field.extras['discriminator'] = discriminator
+                        data_type.parent.data_types.remove(data_type)
+                        data_type.parent.data_types.append(copied_data_type)
+
                     elif isinstance(data_type.parent, DataType):
                         # for data_type
                         data_type_id = id(data_type)
@@ -850,13 +972,20 @@ class Parser(ABC):
                         ]
                     else:  # pragma: no cover
                         continue
+                    original_field = get_most_of_parent(data_type, DataModelFieldBase)
+                    if original_field:  # pragma: no cover
+                        # TODO: Improve detection of reference type
+                        imports.append(original_field.imports)
 
                     data_type.remove_reference()
 
                     root_type_model.reference.children = [
-                        c for c in root_type_model.reference.children if c.parent
+                        c
+                        for c in root_type_model.reference.children
+                        if getattr(c, 'parent', None)
                     ]
 
+                    imports.remove_referenced_imports(root_type_model.path)
                     if not root_type_model.reference.children:
                         unused_models.append(root_type_model)
 
@@ -957,9 +1086,10 @@ class Parser(ABC):
         model_class_name_baseclasses: Dict[DataModel, Tuple[str, Set[str]]] = {}
         for model in models:
             class_name = model.class_name
-            model_class_name_baseclasses[model] = class_name, {
-                b.type_hint for b in model.base_classes if b.reference
-            } - {class_name}
+            model_class_name_baseclasses[model] = (
+                class_name,
+                {b.type_hint for b in model.base_classes if b.reference} - {class_name},
+            )
 
         changed: bool = True
         while changed:
@@ -986,6 +1116,30 @@ class Parser(ABC):
                 if model_field.nullable is not True:  # pragma: no cover
                     model_field.nullable = False
 
+    def __change_imported_model_name(
+        self,
+        models: List[DataModel],
+        imports: Imports,
+        scoped_model_resolver: ModelResolver,
+    ) -> None:
+        imported_names = {
+            imports.alias[from_][i]
+            if i in imports.alias[from_] and i != imports.alias[from_][i]
+            else i
+            for from_, import_ in imports.items()
+            for i in import_
+        }
+        for model in models:
+            if model.class_name not in imported_names:  # pragma: no cover
+                continue
+
+            model.reference.name = scoped_model_resolver.add(  # pragma: no cover
+                path=get_special_path('imported_name', model.path.split('/')),
+                original_name=model.reference.name,
+                unique=True,
+                class_name=True,
+            ).name
+
     def parse(
         self,
         with_import: Optional[bool] = True,
@@ -1004,6 +1158,9 @@ class Parser(ABC):
                 settings_path,
                 self.wrap_string_literal,
                 skip_string_normalization=not self.use_double_quotes,
+                known_third_party=self.known_third_party,
+                custom_formatters=self.custom_formatter,
+                custom_formatters_kwargs=self.custom_formatters_kwargs,
             )
         else:
             code_formatter = None
@@ -1025,14 +1182,15 @@ class Parser(ABC):
 
         module_models: List[Tuple[Tuple[str, ...], List[DataModel]]] = []
         unused_models: List[DataModel] = []
-        model_to_models: Dict[DataModel, List[DataModel]] = {}
+        model_to_module_models: Dict[
+            DataModel, Tuple[Tuple[str, ...], List[DataModel]]
+        ] = {}
+        module_to_import: Dict[Tuple[str, ...], Imports] = {}
 
         previous_module = ()  # type: Tuple[str, ...]
-        for module, models in (
-            (k, [*v]) for k, v in grouped_models
-        ):  # type: Tuple[str, ...], List[DataModel]
+        for module, models in ((k, [*v]) for k, v in grouped_models):  # type: Tuple[str, ...], List[DataModel]
             for model in models:
-                model_to_models[model] = models
+                model_to_module_models[model] = module, models
             self.__delete_duplicate_models(models)
             self.__replace_duplicate_name_in_module(models)
             if len(previous_module) - len(module) > 1:
@@ -1056,9 +1214,12 @@ class Parser(ABC):
             models: List[DataModel]
             init: bool
             imports: Imports
+            scoped_model_resolver: ModelResolver
 
         processed_models: List[Processed] = []
+
         for module, models in module_models:
+            imports = module_to_import[module] = Imports()
             init = False
             if module:
                 parent = (*module[:-1], '__init__.py')
@@ -1072,7 +1233,6 @@ class Parser(ABC):
             else:
                 module = ('__init__.py',)
 
-            imports = Imports()
             scoped_model_resolver = ModelResolver()
 
             self.__replace_unique_list_to_set(models)
@@ -1080,19 +1240,29 @@ class Parser(ABC):
             self.__extract_inherited_enum(models)
             self.__set_reference_default_value_to_field(models)
             self.__reuse_model(models, require_update_action_models)
-            self.__collapse_root_models(models, unused_models)
+            self.__collapse_root_models(models, unused_models, imports)
             self.__set_default_enum_member(models)
             self.__override_required_field(models)
             self.__sort_models(models, imports)
             self.__set_one_literal_on_default(models)
+            self.__apply_discriminator_type(models, imports)
 
-            processed_models.append(Processed(module, models, init, imports))
+            processed_models.append(
+                Processed(module, models, init, imports, scoped_model_resolver)
+            )
 
         for unused_model in unused_models:
-            if unused_model in model_to_models[unused_model]:  # pragma: no cover
-                model_to_models[unused_model].remove(unused_model)
+            module, models = model_to_module_models[unused_model]
+            if unused_model in models:  # pragma: no cover
+                imports = module_to_import[module]
+                imports.remove(unused_model.imports)
+                models.remove(unused_model)
 
-        for module, models, init, imports in processed_models:
+        for module, models, init, imports, scoped_model_resolver in processed_models:
+            # process after removing unused models
+            self.__change_imported_model_name(models, imports, scoped_model_resolver)
+
+        for module, models, init, imports, scoped_model_resolver in processed_models:
             result: List[str] = []
             if with_import:
                 result += [str(self.imports), str(imports), '\n']
