@@ -44,9 +44,11 @@ from datamodel_code_generator.model.base import UNDEFINED, get_module_name
 from datamodel_code_generator.model.enum import Enum
 from datamodel_code_generator.parser import DefaultPutDict, LiteralType
 from datamodel_code_generator.parser.base import (
+    SPECIAL_PATH_FORMAT,
     Parser,
     Source,
     escape_characters,
+    get_special_path,
     title_to_class_name,
 )
 from datamodel_code_generator.reference import ModelType, Reference, is_url
@@ -92,13 +94,6 @@ def get_model_by_path(
     )
 
 
-SPECIAL_PATH_FORMAT: str = '#-datamodel-code-generator-#-{}-#-special-#'
-
-
-def get_special_path(keyword: str, path: List[str]) -> List[str]:
-    return [*path, SPECIAL_PATH_FORMAT.format(keyword)]
-
-
 json_schema_data_formats: Dict[str, Dict[str, Types]] = {
     'integer': {
         'int32': Types.int32,
@@ -173,6 +168,10 @@ class JsonSchemaObject(BaseModel):
             def get_fields(cls) -> Dict[str, Any]:
                 return cls.__fields__
 
+            @classmethod
+            def model_rebuild(cls) -> None:
+                cls.update_forward_refs()
+
     __constraint_fields__: Set[str] = {
         'exclusiveMinimum',
         'minimum',
@@ -189,9 +188,9 @@ class JsonSchemaObject(BaseModel):
     __extra_key__: str = SPECIAL_PATH_FORMAT.format('extras')
 
     @model_validator(mode='before')
-    def validate_exclusive_maximum_and_exclusive_minimum(
-        cls, values: Dict[str, Any]
-    ) -> Any:
+    def validate_exclusive_maximum_and_exclusive_minimum(cls, values: Any) -> Any:
+        if not isinstance(values, dict):
+            return values
         exclusive_maximum: Union[float, bool, None] = values.get('exclusiveMaximum')
         exclusive_minimum: Union[float, bool, None] = values.get('exclusiveMinimum')
 
@@ -238,6 +237,7 @@ class JsonSchemaObject(BaseModel):
     allOf: List[JsonSchemaObject] = []
     enum: List[Any] = []
     writeOnly: Optional[bool] = None
+    readOnly: Optional[bool] = None
     properties: Optional[Dict[str, Union[JsonSchemaObject, bool]]] = None
     required: List[str] = []
     ref: Optional[str] = Field(default=None, alias='$ref')
@@ -270,6 +270,8 @@ class JsonSchemaObject(BaseModel):
         def __init__(self, **data: Any) -> None:
             super().__init__(**data)
             self.extras = {k: v for k, v in data.items() if k not in EXCLUDE_FIELD_KEYS}
+            if 'const' in data.get(self.__extra_key__, {}):
+                self.extras['const'] = data[self.__extra_key__]['const']
 
     @cached_property
     def is_object(self) -> bool:
@@ -332,14 +334,11 @@ def _get_type(type_: str, format__: Optional[str] = None) -> Types:
     if data_formats is not None:
         return data_formats
 
-    warn(
-        'format of {!r} not understood for {!r} - using default'
-        ''.format(format__, type_)
-    )
+    warn(f'format of {format__!r} not understood for {type_!r} - using default' '')
     return json_schema_data_formats[type_]['default']
 
 
-JsonSchemaObject.update_forward_refs()
+JsonSchemaObject.model_rebuild()
 
 DEFAULT_FIELD_KEYS: Set[str] = {
     'example',
@@ -351,7 +350,16 @@ DEFAULT_FIELD_KEYS: Set[str] = {
     'default_factory',
 }
 
-EXCLUDE_FIELD_KEYS = (set(JsonSchemaObject.get_fields()) - DEFAULT_FIELD_KEYS) | {
+EXCLUDE_FIELD_KEYS_IN_JSON_SCHEMA: Set[str] = {
+    'readOnly',
+    'writeOnly',
+}
+
+EXCLUDE_FIELD_KEYS = (
+    set(JsonSchemaObject.get_fields())
+    - DEFAULT_FIELD_KEYS
+    - EXCLUDE_FIELD_KEYS_IN_JSON_SCHEMA
+) | {
     '$id',
     '$ref',
     JsonSchemaObject.__extra_key__,
@@ -360,7 +368,8 @@ EXCLUDE_FIELD_KEYS = (set(JsonSchemaObject.get_fields()) - DEFAULT_FIELD_KEYS) |
 
 @snooper_to_methods(max_variable_length=None)
 class JsonSchemaParser(Parser):
-    SCHEMA_PATH: ClassVar[str] = '#/definitions'
+    SCHEMA_PATHS: ClassVar[List[str]] = ['#/definitions', '#/$defs']
+    SCHEMA_OBJECT_TYPE: ClassVar[Type[JsonSchemaObject]] = JsonSchemaObject
 
     def __init__(
         self,
@@ -371,6 +380,7 @@ class JsonSchemaParser(Parser):
         data_type_manager_type: Type[DataTypeManager] = pydantic_model.DataTypeManager,
         data_model_field_type: Type[DataModelFieldBase] = pydantic_model.DataModelField,
         base_class: Optional[str] = None,
+        additional_imports: Optional[List[str]] = None,
         custom_template_dir: Optional[Path] = None,
         extra_template_data: Optional[DefaultDict[str, Dict[str, Any]]] = None,
         target_python_version: PythonVersion = PythonVersion.PY_37,
@@ -424,6 +434,9 @@ class JsonSchemaParser(Parser):
         remove_special_field_name_prefix: bool = False,
         capitalise_enum_members: bool = False,
         keep_model_order: bool = False,
+        known_third_party: Optional[List[str]] = None,
+        custom_formatters: Optional[List[str]] = None,
+        custom_formatters_kwargs: Optional[Dict[str, Any]] = None,
     ) -> None:
         super().__init__(
             source=source,
@@ -432,6 +445,7 @@ class JsonSchemaParser(Parser):
             data_type_manager_type=data_type_manager_type,
             data_model_field_type=data_model_field_type,
             base_class=base_class,
+            additional_imports=additional_imports,
             custom_template_dir=custom_template_dir,
             extra_template_data=extra_template_data,
             target_python_version=target_python_version,
@@ -485,6 +499,9 @@ class JsonSchemaParser(Parser):
             remove_special_field_name_prefix=remove_special_field_name_prefix,
             capitalise_enum_members=capitalise_enum_members,
             keep_model_order=keep_model_order,
+            known_third_party=known_third_party,
+            custom_formatters=custom_formatters,
+            custom_formatters_kwargs=custom_formatters_kwargs,
         )
 
         self.remote_object_cache: DefaultPutDict[str, Dict[str, Any]] = DefaultPutDict()
@@ -498,26 +515,34 @@ class JsonSchemaParser(Parser):
             *self.field_extra_keys_without_x_prefix,
         }
 
+        if self.data_model_field_type.can_have_extra_keys:
+            self.get_field_extra_key: Callable[[str], str] = (
+                lambda key: self.model_resolver.get_valid_field_name_and_alias(key)[0]
+            )
+
+        else:
+            self.get_field_extra_key = lambda key: key
+
     def get_field_extras(self, obj: JsonSchemaObject) -> Dict[str, Any]:
         if self.field_include_all_keys:
             return {
-                self.model_resolver.get_valid_field_name_and_alias(
+                self.get_field_extra_key(
                     k.lstrip('x-') if k in self.field_extra_keys_without_x_prefix else k
-                )[0]: v
+                ): v
                 for k, v in obj.extras.items()
             }
         else:
             return {
-                self.model_resolver.get_valid_field_name_and_alias(
+                self.get_field_extra_key(
                     k.lstrip('x-') if k in self.field_extra_keys_without_x_prefix else k
-                )[0]: v
+                ): v
                 for k, v in obj.extras.items()
                 if k in self.field_keys
             }
 
     @cached_property
-    def schema_paths(self) -> List[str]:
-        return self.SCHEMA_PATH.lstrip('#/').split('/')
+    def schema_paths(self) -> List[Tuple[str, List[str]]]:
+        return [(s, s.lstrip('#/').split('/')) for s in self.SCHEMA_PATHS]
 
     @property
     def root_id(self) -> Optional[str]:
@@ -615,45 +640,95 @@ class JsonSchemaParser(Parser):
         if obj.title:
             self.extra_template_data[name]['title'] = obj.title
 
-    def parse_any_of(
-        self, name: str, obj: JsonSchemaObject, path: List[str]
+    def _deep_merge(
+        self, dict1: Dict[Any, Any], dict2: Dict[Any, Any]
+    ) -> Dict[Any, Any]:
+        result = dict1.copy()
+        for key, value in dict2.items():
+            if key in result:
+                if isinstance(result[key], dict) and isinstance(value, dict):
+                    result[key] = self._deep_merge(result[key], value)
+                    continue
+                elif isinstance(result[key], list) and isinstance(value, list):
+                    result[key] = result[key] + value
+                    continue
+            result[key] = value
+        return result
+
+    def parse_combined_schema(
+        self,
+        name: str,
+        obj: JsonSchemaObject,
+        path: List[str],
+        target_attribute_name: str,
     ) -> List[DataType]:
-        data_types = self.parse_list_item(name, obj.anyOf, path, obj)
-        if not obj.properties and not obj.required:
-            return data_types
+        base_object = obj.dict(
+            exclude={target_attribute_name}, exclude_unset=True, by_alias=True
+        )
+        combined_schemas: List[JsonSchemaObject] = []
+        refs = []
+        for index, target_attribute in enumerate(
+            getattr(obj, target_attribute_name, [])
+        ):
+            if target_attribute.ref:
+                combined_schemas.append(target_attribute)
+                refs.append(index)
+                # TODO: support partial ref
+                # {
+                #   "type": "integer",
+                #   "oneOf": [
+                #     { "minimum": 5 },
+                #     { "$ref": "#/definitions/positive" }
+                #   ],
+                #    "definitions": {
+                #     "positive": {
+                #       "minimum": 0,
+                #       "exclusiveMinimum": true
+                #     }
+                #    }
+                # }
+            else:
+                combined_schemas.append(
+                    self.SCHEMA_OBJECT_TYPE.parse_obj(
+                        self._deep_merge(
+                            base_object,
+                            target_attribute.dict(exclude_unset=True, by_alias=True),
+                        )
+                    )
+                )
+
+        parsed_schemas = self.parse_list_item(
+            name,
+            combined_schemas,
+            path,
+            obj,
+            singular_name=False,
+        )
+        common_path_keyword = f'{target_attribute_name}Common'
         return [
             self._parse_object_common_part(
                 name,
                 obj,
-                [*get_special_path('anyOfCommon', path), str(i)],
+                [*get_special_path(common_path_keyword, path), str(i)],
                 ignore_duplicate_model=True,
                 fields=[],
                 base_classes=[d.reference],
                 required=[],
             )
-            for i, d in enumerate(data_types)
-            if d.reference
+            if i in refs and d.reference
+            else d
+            for i, d in enumerate(parsed_schemas)
         ]
 
+    def parse_any_of(
+        self, name: str, obj: JsonSchemaObject, path: List[str]
+    ) -> List[DataType]:
+        return self.parse_combined_schema(name, obj, path, 'anyOf')
+
     def parse_one_of(
         self, name: str, obj: JsonSchemaObject, path: List[str]
     ) -> List[DataType]:
-        data_types = self.parse_list_item(name, obj.oneOf, path, obj)
-        if not obj.properties and not obj.required:
-            return data_types
-        return [
-            self._parse_object_common_part(
-                name,
-                obj,
-                [*get_special_path('oneOfCommon', path), str(i)],
-                ignore_duplicate_model=True,
-                fields=[],
-                base_classes=[d.reference],
-                required=[],
-            )
-            for i, d in enumerate(data_types)
-            if d.reference
-        ]
+        return self.parse_combined_schema(name, obj, path, 'oneOf')
 
     def _parse_object_common_part(
         self,
@@ -671,17 +746,30 @@ class JsonSchemaParser(Parser):
             )
         # ignore an undetected object
         if ignore_duplicate_model and not fields and len(base_classes) == 1:
-            self.model_resolver.delete(path)
-            return self.data_type(reference=base_classes[0])
+            with self.model_resolver.current_base_path_context(
+                self.model_resolver._base_path
+            ):
+                self.model_resolver.delete(path)
+                return self.data_type(reference=base_classes[0])
         if required:
             for field in fields:
+                if self.force_optional_for_required_fields or (
+                    self.apply_default_values_for_required_fields and field.has_default
+                ):
+                    continue
                 if (field.original_name or field.name) in required:
                     field.required = True
         if obj.required:
             field_name_to_field = {f.original_name or f.name: f for f in fields}
             for required_ in obj.required:
                 if required_ in field_name_to_field:
-                    field_name_to_field[required_].required = True
+                    field = field_name_to_field[required_]
+                    if self.force_optional_for_required_fields or (
+                        self.apply_default_values_for_required_fields
+                        and field.has_default
+                    ):
+                        continue
+                    field.required = True
                 else:
                     fields.append(
                         self.data_model_field_type(
@@ -742,12 +830,14 @@ class JsonSchemaParser(Parser):
                     union_models,
                 )
                 if all_of_item.anyOf:
+                    self.model_resolver.add(path, name, class_name=True, loaded=True)
                     union_models.extend(
                         d.reference
                         for d in self.parse_any_of(name, all_of_item, path)
                         if d.reference
                     )
                 if all_of_item.oneOf:
+                    self.model_resolver.add(path, name, class_name=True, loaded=True)
                     union_models.extend(
                         d.reference
                         for d in self.parse_one_of(name, all_of_item, path)
@@ -1010,6 +1100,13 @@ class JsonSchemaParser(Parser):
             return self.parse_array_fields(
                 name, item, get_special_path('array', path)
             ).data_type
+        elif (
+            item.discriminator
+            and parent
+            and parent.is_array
+            and (item.oneOf or item.anyOf)
+        ):
+            return self.parse_root_type(name, item, path)
         elif item.anyOf:
             return self.data_type(
                 data_types=self.parse_any_of(
@@ -1024,6 +1121,7 @@ class JsonSchemaParser(Parser):
             )
         elif item.allOf:
             all_of_path = get_special_path('allOf', path)
+            all_of_path = [self.model_resolver.resolve_ref(all_of_path)]
             return self.parse_all_of(
                 self.model_resolver.add(
                     all_of_path, name, singular_name=singular_name, class_name=True
@@ -1131,7 +1229,6 @@ class JsonSchemaParser(Parser):
             data_types.append(
                 self.parse_enum(name, obj, get_special_path('enum', path))
             )
-
         return self.data_model_field_type(
             data_type=self.data_type(data_types=data_types),
             default=obj.default,
@@ -1202,6 +1299,7 @@ class JsonSchemaParser(Parser):
         obj: JsonSchemaObject,
         path: List[str],
     ) -> DataType:
+        reference: Optional[Reference] = None
         if obj.ref:
             data_type: DataType = self.get_ref_data_type(obj.ref)
         elif obj.custom_type_path:
@@ -1209,14 +1307,16 @@ class JsonSchemaParser(Parser):
                 obj.custom_type_path, is_custom_type=True
             )
         elif obj.anyOf or obj.oneOf:
-            object_path = [*path, name]
+            reference = self.model_resolver.add(
+                path, name, loaded=True, class_name=True
+            )
             if obj.anyOf:
                 data_types: List[DataType] = self.parse_any_of(
-                    name, obj, get_special_path('anyOf', object_path)
+                    name, obj, get_special_path('anyOf', path)
                 )
             else:
                 data_types = self.parse_one_of(
-                    name, obj, get_special_path('oneOf', object_path)
+                    name, obj, get_special_path('oneOf', path)
                 )
 
             if len(data_types) > 1:
@@ -1246,7 +1346,10 @@ class JsonSchemaParser(Parser):
             )
         if self.use_title_as_name and obj.title:
             name = obj.title
-        reference = self.model_resolver.add(path, name, loaded=True, class_name=True)
+        if not reference:
+            reference = self.model_resolver.add(
+                path, name, loaded=True, class_name=True
+            )
         self.set_title(name, obj)
         self.set_additional_properties(name, obj)
         data_model_root_type = self.data_model_root_type(
@@ -1532,7 +1635,7 @@ class JsonSchemaParser(Parser):
         raw: Dict[str, Any],
         path: List[str],
     ) -> None:
-        self.parse_obj(name, JsonSchemaObject.parse_obj(raw), path)
+        self.parse_obj(name, self.SCHEMA_OBJECT_TYPE.parse_obj(raw), path)
 
     def parse_obj(
         self,
@@ -1659,24 +1762,33 @@ class JsonSchemaParser(Parser):
                 # Some jsonschema docs include attribute self to have include version details
                 raw.pop('self', None)
                 # parse $id before parsing $ref
-                root_obj = JsonSchemaObject.parse_obj(raw)
+                root_obj = self.SCHEMA_OBJECT_TYPE.parse_obj(raw)
                 self.parse_id(root_obj, path_parts)
-                try:
-                    definitions = get_model_by_path(raw, self.schema_paths)
-                except KeyError:
+                definitions: Optional[Dict[Any, Any]] = None
+                for schema_path, split_schema_path in self.schema_paths:
+                    try:
+                        definitions = get_model_by_path(raw, split_schema_path)
+                        if definitions:
+                            break
+                    except KeyError:
+                        continue
+                if definitions is None:
                     definitions = {}
+
                 for key, model in definitions.items():
-                    obj = JsonSchemaObject.parse_obj(model)
-                    self.parse_id(obj, [*path_parts, self.SCHEMA_PATH, key])
+                    obj = self.SCHEMA_OBJECT_TYPE.parse_obj(model)
+                    self.parse_id(obj, [*path_parts, schema_path, key])
 
                 if object_paths:
                     models = get_model_by_path(raw, object_paths)
                     model_name = object_paths[-1]
-                    self.parse_obj(model_name, JsonSchemaObject.parse_obj(models), path)
+                    self.parse_obj(
+                        model_name, self.SCHEMA_OBJECT_TYPE.parse_obj(models), path
+                    )
                 else:
                     self.parse_obj(obj_name, root_obj, path_parts or ['#'])
                 for key, model in definitions.items():
-                    path = [*path_parts, self.SCHEMA_PATH, key]
+                    path = [*path_parts, schema_path, key]
                     reference = self.model_resolver.get(path)
                     if not reference or not reference.loaded:
                         self.parse_raw_obj(key, model, path)
@@ -1693,7 +1805,7 @@ class JsonSchemaParser(Parser):
                         models = get_model_by_path(raw, object_paths)
                         model_name = object_paths[-1]
                         self.parse_obj(
-                            model_name, JsonSchemaObject.parse_obj(models), path
+                            model_name, self.SCHEMA_OBJECT_TYPE.parse_obj(models), path
                         )
                     previous_reserved_refs = reserved_refs
                     reserved_refs = set(self.reserved_refs.get(key) or [])
