@@ -1,5 +1,3 @@
-from __future__ import annotations
-
 import re
 from abc import ABC, abstractmethod
 from enum import Enum, auto
@@ -110,14 +108,17 @@ class UnionIntFloat:
     def __float__(self) -> float:
         return float(self.value)
 
+    def __str__(self) -> str:
+        return str(self.value)
+
     @classmethod
     def __get_validators__(cls) -> Iterator[Callable[[Any], Any]]:
         yield cls.validate
 
     @classmethod
     def __get_pydantic_core_schema__(
-        cls, _source_type: Any, _handler: GetCoreSchemaHandler
-    ) -> core_schema.CoreSchema:
+        cls, _source_type: Any, _handler: 'GetCoreSchemaHandler'
+    ) -> 'core_schema.CoreSchema':
         from_int_schema = core_schema.chain_schema(
             [
                 core_schema.union_schema(
@@ -128,7 +129,7 @@ class UnionIntFloat:
         )
 
         return core_schema.json_or_python_schema(
-            json_schema=core_schema.no_info_plain_validator_function(cls.validate),
+            json_schema=from_int_schema,
             python_schema=core_schema.union_schema(
                 [
                     # check if it's an instance first before doing any further work
@@ -142,10 +143,21 @@ class UnionIntFloat:
         )
 
     @classmethod
-    def validate(cls, v: Any) -> UnionIntFloat:
+    def validate(cls, v: Any) -> 'UnionIntFloat':
         if isinstance(v, UnionIntFloat):
             return v
         elif not isinstance(v, (int, float)):  # pragma: no cover
+            try:
+                int(v)
+                return cls(v)
+            except (TypeError, ValueError):
+                pass
+            try:
+                float(v)
+                return cls(v)
+            except (TypeError, ValueError):
+                pass
+
             raise TypeError(f'{v} is not int or float')
         return cls(v)
 
@@ -234,6 +246,11 @@ class DataType(_BaseModel):
             revalidate_instances='never',
         )
     else:
+        if not TYPE_CHECKING:
+
+            @classmethod
+            def model_rebuild(cls) -> None:
+                cls.update_forward_refs()
 
         class Config:
             extra = 'forbid'
@@ -245,7 +262,7 @@ class DataType(_BaseModel):
 
     type: Optional[str] = None
     reference: Optional[Reference] = None
-    data_types: List[DataType] = []
+    data_types: List['DataType'] = []
     is_func: bool = False
     kwargs: Optional[Dict[str, Any]] = None
     import_: Optional[Import] = None
@@ -263,14 +280,14 @@ class DataType(_BaseModel):
     parent: Optional[Any] = None
     children: List[Any] = []
     strict: bool = False
-    dict_key: Optional[DataType] = None
+    dict_key: Optional['DataType'] = None
 
     _exclude_fields: ClassVar[Set[str]] = {'parent', 'children'}
     _pass_fields: ClassVar[Set[str]] = {'parent', 'children', 'data_types', 'reference'}
 
     @classmethod
     def from_import(
-        cls: Type[DataTypeT],
+        cls: Type['DataTypeT'],
         import_: Import,
         *,
         is_optional: bool = False,
@@ -280,7 +297,7 @@ class DataType(_BaseModel):
         is_custom_type: bool = False,
         strict: bool = False,
         kwargs: Optional[Dict[str, Any]] = None,
-    ) -> DataTypeT:
+    ) -> 'DataTypeT':
         return cls(
             type=import_.import_,
             import_=import_,
@@ -337,7 +354,7 @@ class DataType(_BaseModel):
         return self.reference.short_name  # type: ignore
 
     @property
-    def all_data_types(self) -> Iterator[DataType]:
+    def all_data_types(self) -> Iterator['DataType']:
         for data_type in self.data_types:
             yield from data_type.all_data_types
         yield self
@@ -428,10 +445,19 @@ class DataType(_BaseModel):
                     data_type_type = data_type.type_hint
                     if data_type_type in data_types:  # pragma: no cover
                         continue
-                    data_types.append(data_type_type)
-                if NONE in data_types:
-                    data_types = [d for d in data_types if d != NONE]
-                    self.is_optional = True
+
+                    if NONE == data_type_type:
+                        self.is_optional = True
+                        continue
+
+                    non_optional_data_type_type = _remove_none_from_union(
+                        data_type_type, self.use_union_operator
+                    )
+
+                    if non_optional_data_type_type != data_type_type:
+                        self.is_optional = True
+
+                    data_types.append(non_optional_data_type_type)
                 if len(data_types) == 1:
                     type_ = data_types[0]
                 else:
@@ -498,7 +524,7 @@ class DataType(_BaseModel):
         return len(self.data_types) > 1
 
 
-DataType.update_forward_refs()
+DataType.model_rebuild()
 
 DataTypeT = TypeVar('DataTypeT', bound=DataType)
 
